# Generated by Django 2.1.7 on 2019-02-20 03:26

from django.db import migrations


def create_default_templates(apps, schema_editor):
    MyObject = apps.get_model(
        'demoapp', 'MyObject')

    objs = [
        MyObject(
            name="OBJ 1",
            limit_quantity=10.7,
            measurement_unit='1'
        ),
        MyObject(
            name="OBJ 2",
            limit_quantity=40.1,
            measurement_unit='2'
        ),
        MyObject(
            name="OBJ 3",
            limit_quantity=107,
            measurement_unit='3'
        ),
        MyObject(
            name="OBJ 4",
            limit_quantity=10.7,
            measurement_unit='4'
        ),
        MyObject(
            name="OBJ 5",
            limit_quantity=10.7,
            measurement_unit='2'
        ),
        MyObject(
            name="OBJ 6",
            limit_quantity=72,
            measurement_unit='1'
        )
    ]
    MyObject.objects.bulk_create(objs)


class Migration(migrations.Migration):

    dependencies = [
        ('demoapp', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(create_default_templates),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-05 19:08
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djreservation', '0005_reservationtoken'),
    ]

    operations = [
        migrations.AddField(
            model_name='reservationtoken',
            name='base_url',
            field=models.URLField(default='http://localhost:8000'),
        ),
    ]



# encoding: utf-8

'''
Free as freedom will be 2/9/2016

@author: luisza
'''

from __future__ import unicode_literals
from djreservation.views import ProductReservationView
from django.conf.urls import url


class ModelsRegistered(object):
    views = []

    def register(self,
                 base_model,
                 amount_field,
                 extra_display_field=[],
                 view=ProductReservationView
                 ):

        self.views.append(
            view(
                base_model,
                amount_field,
                extra_display_field
            )
        )

    def get_urls(self):
        dev = []

        for i, view in enumerate(self.views):
            appname = view.base_model._meta.app_label
            modelname = view.base_model.__class__.__name__
            dev.append(
                url("regProduct/%s/%s/(?P<modelpk>)\d+$" % (
                    appname, modelname),
                    self.views[i].as_view(),
                    name="reserve_product_%s_%s" % (
                    appname, modelname))

            )
        return dev


register_product = ModelsRegistered()



from django.db import models
from django.utils.translation import gettext_lazy as _


class MyObject(models.Model):
    CHOICES = (
        ('0', _('Meters')),
        ('1', _('Milimeters')),
        ('2', _('Centimeters')),
        ('3', _('Liters')),
        ('4', _('Mililiters')),
        ('5', _('Unit'))
    )
    name = models.CharField(max_length=200)
    quantity = models.FloatField()
    measurement_unit = models.CharField(max_length=2, choices=CHOICES)

    def __str__(self):
        return self.name



# encoding: utf-8

'''
Free as freedom will be 3/9/2016

@author: luisza
'''

from __future__ import unicode_literals
from .models import Reservation
from django.core.mail import send_mail
from django.utils.translation import gettext_lazy as _
from django.template.loader import render_to_string
from django.conf import settings

BUILDING = Reservation.BUILDING
REQUESTED = Reservation.REQUESTED
ACCEPTED = Reservation.ACCEPTED
DENIED = Reservation.DENIED
BORROWED = Reservation.BORROWED
RETURNED = Reservation.RETURNED


def send_html_email(subject, template, context=None):
    if context is None:
        context = {}
    reservation = context['reservation']
    message = render_to_string(template,
                               context)
    send_mail(
        subject=subject,
        message=_('Please, use an email with html support'),
        from_email=settings.DEFAULT_FROM_EMAIL,
        recipient_list=[reservation.user.email],
        fail_silently=True,
        html_message=message
    )


def email_requested(reservation, user):
    send_html_email(
        _('Reservation requested: reserv-%d') % (reservation.pk),
        'djreservation/mail/requested.txt',
        context={
            'reservation': reservation,
            'user': user
        })


def email_accepted(reservation, user):
    send_html_email(
        subject=_('Reservation accepted: reserv-%d') % (reservation.pk),
        template='djreservation/mail/accepted.txt',
        context={
            'reservation': reservation,
            'user': user
        })


def email_denied(reservation, user):
    send_html_email(
        _('Reservation denied: reserv-%d') % (reservation.pk),
        'djreservation/mail/denied.txt',
        context={
            'reservation': reservation,
            'user': user
        })


def email_borrowed(reservation, user):
    send_html_email(
        _('Reservation borrowed: reserv-%d') % (reservation.pk),
        'djreservation/mail/borrowed.txt',
        context={
            'reservation': reservation,
            'user': user
        })


def email_returned(reservation, user):
    send_html_email(
        _('Reservation returned: reserv-%d') % (reservation.pk),
        'djreservation/mail/returned.txt',
        context={
            'reservation': reservation,
            'user': user
        })


def email_building(reservation, user):
    pass


email_functions = {
    BUILDING: email_building,
    REQUESTED: email_requested,
    ACCEPTED: email_accepted,
    DENIED: email_denied,
    BORROWED: email_borrowed,
    RETURNED: email_returned
}


def send_reservation_email(reservation, user):
    email_functions[reservation.status](reservation, user)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-05 17:37
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('djreservation', '0004_auto_20161005_0022'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReservationToken',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('token', models.UUIDField(default=uuid.uuid4, editable=False)),
                ('reservation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djreservation.Reservation')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-04 21:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djreservation', '0002_auto_20160903_0030'),
    ]

    operations = [
        migrations.AlterField(
            model_name='reservation',
            name='status',
            field=models.SmallIntegerField(choices=[(0, 'Building'), (1, 'Requested'), (2, 'Acepted'), (3, 'Denied'), (4, 'Borrowed'), (5, 'Returned')], default=0),
        ),
    ]



# Generated by Django 2.1.7 on 2019-02-20 04:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('demoapp', '0002_import_example_data'),
    ]

    operations = [
        migrations.RenameField(
            model_name='myobject',
            old_name='limit_quantity',
            new_name='quantity',
        ),
    ]



# encoding: utf-8


'''
Created on 10/12/2016

@author: luisza
'''
from __future__ import unicode_literals
from djreservation.models import Reservation


def proccess_reservation(obj, differ_obj, change_status):
    if obj.status == Reservation.BUILDING or obj.status == Reservation.REQUESTED:
        return
    if obj.status == Reservation.ACCEPTED:
        accepted_reservation(obj, differ_obj, change_status)
    elif obj.status == Reservation.DENIED:
        denied_reservation(obj, differ_obj, change_status)
    elif obj.status == Reservation.BORROWED:
        borrowed_reservation(obj, differ_obj, change_status)
    elif obj.status == Reservation.RETURNED:
        returned_reservation(obj, differ_obj, change_status)


def denied_reservation(instance, differ_obj, change_status):
    pass


def accepted_reservation(instance, differ_obj, change_status):
    pass


def borrowed_reservation(instance, differ_obj, change_status):
    not_borrowed = []
    if change_status:
        query = instance.product_set.filter(borrowed=True)
    else:
        query = instance.product_set.filter(pk__in=differ_obj, borrowed=True)
        not_borrowed = instance.product_set.filter(
            pk__in=differ_obj, borrowed=False)
    for product in query:
        ref_obj = product.content_object
        setattr(ref_obj, product.amount_field,
                getattr(ref_obj, product.amount_field) - product.amount)
        ref_obj.save()

    for product in not_borrowed:
        ref_obj = product.content_object
        setattr(ref_obj, product.amount_field,
                getattr(ref_obj, product.amount_field) + product.amount)
        ref_obj.save()


def returned_reservation(instance, differ_obj, change_status):

    if change_status:
        query = instance.product_set.filter(borrowed=True)
    else:
        query = instance.product_set.filter(pk_in=differ_obj, borrowed=True)

    for product in query:
        ref_obj = product.content_object
        setattr(ref_obj, product.amount_field, getattr(
            ref_obj, product.amount_field) + product.amount)
        ref_obj.save()



# encoding: utf-8

'''
Free as freedom will be 5/10/2016

@author: luisza
'''
from __future__ import unicode_literals

from django.db.models.signals import post_save, pre_save
from django.dispatch import receiver

from djreservation.email import send_reservation_email
from djreservation.models import Reservation


@receiver(pre_save, sender=Reservation)
def update_product_related(sender, **kwargs):
    instance = kwargs['instance']
    if instance.pk is None:
        return
    status = sender.objects.filter(
        pk=instance.pk).values('status')[0]['status']
    if instance.status == status:
        return

    send_reservation_email(instance, instance.user)


from django.apps import AppConfig


class DjreservationConfig(AppConfig):
    name = 'djreservation'
    default_auto_field = 'django.db.models.BigAutoField'

    def ready(self):
        import djreservation.signals
        AppConfig.ready(self)


'''
Created on 1/8/2016

@author: luisza
'''

from django.urls import path, re_path
from . import views
from .settings import TOKENIZE

urlpatterns = [
    path("reservation/create", views.CreateReservation.as_view(), name="add_user_reservation"),
    path("reservation/finish", views.finish_reservation, name="finish_reservation"),
    path("reservation/delete_product_reservation/<int:pk>/", views.deleteProduct, name="delete_product_reservation"),
    path("reservation/list", views.ReservationList.as_view(), name="reservation_list"),
]

if TOKENIZE:
    urlpatterns += [
        re_path(r"reservation/token/(?P<pk>\d+)/(?P<token>[0-9a-f-]+)/(?P<status>\d)$",
            views.update_reservation_by_token,
            name="reservation_token")
    ]



# Generated by Django 2.1.7 on 2019-02-20 03:25

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='MyObject',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('limit_quantity', models.FloatField()),
                ('measurement_unit', models.CharField(choices=[('0', 'Meters'), ('1', 'Milimeters'), ('2', 'Centimeters'), ('3', 'Liters'), ('4', 'Mililiters'), ('5', 'Unit')], max_length=2)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-09-03 02:40
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Product',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('amount', models.FloatField()),
                ('amount_field', models.CharField(max_length=150)),
                ('borrowed', models.BooleanField(default=False)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
            ],
        ),
        migrations.CreateModel(
            name='Reservation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reserved_start_date', models.DateTimeField(default=django.utils.timezone.now)),
                ('return_date', models.DateTimeField()),
                ('status', models.SmallIntegerField(choices=[(0, 'building'), (1, 'Requested'), (2, 'Acepted'), (3, 'Denied'), (4, 'Returned')], default=0)),
                ('updated_datetime', models.DateTimeField(auto_now=True)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='product',
            name='reservation',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djreservation.Reservation'),
        ),
    ]



# encoding: utf-8

'''
Free as freedom will be 2/9/2016

@author: luisza
'''

from __future__ import unicode_literals

from django import forms
from django.utils.translation import gettext_lazy as _

from .models import Product, Reservation


class ReservationForm(forms.ModelForm):

    def clean(self):
        if hasattr(self, 'request'):
            if hasattr(self.request, "reservation"):
                raise forms.ValidationError(
                    _("You can not create reservation with active reservation"))
            cleaned_data = super(ReservationForm, self).clean()

    class Meta:
        model = Reservation
        fields = '__all__'
        exclude = ['user', 'status']


class ProductForm(forms.ModelForm):
    model_instance = forms.CharField(widget=forms.HiddenInput)
    available_amount = forms.FloatField(widget=forms.HiddenInput)

    def clean(self):
        cleaned_data = super(ProductForm, self).clean()

        if cleaned_data['amount'] <= 0:
            raise forms.ValidationError(
                _("You amount correct, requested 0 or negative value"))

        if cleaned_data['amount'] > cleaned_data['available_amount']:
            raise forms.ValidationError(
                _("You requested more than product available"))

    class Meta:
        model = Product
        fields = ['amount']



from django.db import models
from django.conf import settings
from django.contrib.auth.models import User
from django.utils import timezone
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.utils.translation import gettext_lazy as _
import uuid

auth_user = settings.AUTH_USER_MODEL if getattr(
    settings, "AUTH_USER_MODEL") else User


class Reservation(models.Model):
    BUILDING = 0
    REQUESTED = 1
    ACCEPTED = 2
    DENIED = 3
    BORROWED = 4
    RETURNED = 5
    STATUS = (
        (BUILDING, _("Building")),
        (REQUESTED, _("Requested")),
        (ACCEPTED, _("Accepted")),
        (DENIED, _("Denied")),
        (BORROWED, _("Borrowed")),
        (RETURNED, _("Returned")),
    )
    user = models.ForeignKey(auth_user, on_delete=models.CASCADE)
    reserved_start_date = models.DateTimeField(default=timezone.now)
    reserved_end_date = models.DateTimeField()
    status = models.SmallIntegerField(choices=STATUS, default=BUILDING)
    updated_datetime = models.DateTimeField(auto_now=True)

    def __str__(self):
        return "%s  %s  (%s to %s)" % (self.user.get_full_name(),
                                       self.get_status_display(),
                                       self.reserved_start_date.strftime(
            "%Y/%m/%d %H:%S"),
            self.reserved_end_date.strftime(
            "%Y/%m/%d %H:%S"),
        )


class Product(models.Model):

    reservation = models.ForeignKey(Reservation,  on_delete=models.CASCADE)
    amount = models.FloatField()
    amount_field = models.CharField(max_length=150)
    borrowed = models.BooleanField(default=False)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    @property
    def available_amount(self):
        return getattr(self.content_object, self.amount_field)

    @property
    def amount_without_this_product(self):
        amount_now = self.available_amount
        return amount_now - self.amount

    def __str__(self):
        return "%.2f ) %s" % (self.amount, self.content_object)


class Observation(models.Model):
    reservation = models.ForeignKey(Reservation, on_delete=models.CASCADE)
    text = models.TextField()


class ReservationToken(models.Model):
    reservation = models.ForeignKey(Reservation, on_delete=models.CASCADE)
    token = models.UUIDField(default=uuid.uuid4, editable=False)
    base_url = models.URLField(default="http://localhost:8000")



# encoding: utf-8

'''
Free as freedom will be 2/9/2016

@author: luisza
'''

from __future__ import unicode_literals
from djreservation.models import Reservation


class ReservationMiddleware(object):

    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.
        if request.COOKIES.get('reservation'):
            try:
                reservation = Reservation.objects.get(
                    pk=request.COOKIES.get('reservation')
                )
                setattr(request, "reservation", reservation)
            except:
                pass

        response = self.get_response(request)

        return response



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-09-03 06:30
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djreservation', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Observation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
            ],
        ),
        migrations.AlterField(
            model_name='reservation',
            name='status',
            field=models.SmallIntegerField(choices=[(0, 'building'), (1, 'Requested'), (2, 'Acepted'), (3, 'Denied'), (4, 'Borrowed'), (5, 'Returned')], default=0),
        ),
        migrations.AddField(
            model_name='observation',
            name='reservation',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djreservation.Reservation'),
        ),
    ]



from django.contrib import admin
from django.contrib.contenttypes.admin import GenericTabularInline
from django.template.loader import render_to_string
from django.utils.html import format_html_join
from django.utils.safestring import mark_safe

from .email import send_reservation_email
from .models import Reservation, Observation, Product
from .product_manager import proccess_reservation

# Register your models here.


class ObservationInline(admin.StackedInline):
    model = Observation
    #fields = '__all__'
    extra = 0


def different(l1, l2):
    l1 = list(l1)
    l2 = list(l2)
    for x in l1:
        if x in l2:
            l1.remove(x)
            l2.remove(x)

    for x in l2:
        if x in l1:
            l1.remove(x)
            l2.remove(x)
    return l1 + l2


class ReservationAdmin(admin.ModelAdmin):
    readonly_fields = ["user", "updated_datetime",
                       "list_of_products"]

    list_display = [
        'user', 'status', "reserved_start_date", "reserved_end_date"]
    list_filter = ['status']
    search_fields = ['user__firstname', "user__lastname"]
    inlines = [ObservationInline]
    date_hierarchy = 'reserved_start_date'
    fieldsets = (
        (None, {
            'fields': (
                ("user", "status", "updated_datetime"),
                ("reserved_start_date", "reserved_end_date"),
                ("list_of_products")
            )
        }),
    )

    def get_readonly_fields(self, request, obj=None):
        readonly =  admin.ModelAdmin.get_readonly_fields(self, request, obj=obj)
        if obj and obj.status == obj.RETURNED:
            readonly= ("user", "status", "updated_datetime",
                "reserved_start_date", "reserved_end_date",
                "list_of_products")
        return readonly

    def get_queryset(self, request):
        queryset = admin.ModelAdmin.get_queryset(self, request)
        queryset = queryset.exclude(status=Reservation.BUILDING)
        return queryset

    def list_of_products(self, instance):

        return mark_safe(render_to_string(
            'djreservation/product_admin_reservation.html',
            {"instance": instance}
        ))

    list_of_products.short_description = "list of products"

    def save_model(self, request, obj, form, change):
        differ_obj = []
        old_status, product_change = -1, False
        if change:
            old_status = obj.__class__.objects.filter(
                pk=obj.pk).values_list('status')[0][0]
        dev = admin.ModelAdmin.save_model(self, request, obj, form, change)
        if 'djreservation_product_list' in request.POST:
            product_pks = request.POST.getlist("djreservation_product_list")
            old_pks = list(map(lambda x: str(x[0]),
                               obj.product_set.all().filter(
                borrowed=True).values_list("pk")))
            differ_obj = different(product_pks, old_pks)
            if any(differ_obj):
                obj.product_set.all().exclude(
                    pk__in=product_pks).update(borrowed=False)

                obj.product_set.all().filter(
                    pk__in=product_pks).update(borrowed=True)
                product_change = True

        change_status = int(old_status) != obj.status
        if product_change or change_status:
            #            send_reservation_email(obj, request.user)
            proccess_reservation(obj, differ_obj, change_status)
        return dev

admin.site.register(Reservation, ReservationAdmin)



# Generated by Django 2.1.7 on 2019-02-20 03:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djreservation', '0006_reservationtoken_base_url'),
    ]

    operations = [
        migrations.AlterField(
            model_name='reservation',
            name='status',
            field=models.SmallIntegerField(choices=[(0, 'Building'), (1, 'Requested'), (2, 'Accepted'), (3, 'Denied'), (4, 'Borrowed'), (5, 'Returned')], default=0),
        ),
    ]



from django.shortcuts import render, get_object_or_404,\
    redirect
from django.views.generic.edit import CreateView
from .models import Product, Reservation, ReservationToken
from .forms import ProductForm, ReservationForm
from django.http.response import HttpResponseRedirect, Http404
from django.urls import reverse

from django.utils.translation import gettext_lazy as _
from .email import send_reservation_email
from django.views.generic.list import ListView

from django.contrib import messages
from .settings import (END_RESERVATION_DATETIME,
                       START_RESERVATION_DATETIME,
                       TOKENIZE)


def get_base_url(request):
    protocol = request.is_secure() and 'https://' or 'http://'
    domain = request.META['HTTP_HOST']
    return protocol + domain


class ReservationList(ListView):
    model = Reservation
    paginate_by = 10

    def get_queryset(self):
        queryset = ListView.get_queryset(self)
        queryset = queryset.filter(user=self.request.user).exclude(
            status=Reservation.BUILDING).order_by('-updated_datetime', 'status')
        return queryset


class CreateReservation(CreateView):
    model = Reservation
    form_class = ReservationForm
    success_url = "/"

    def get_success_url(self):
        if self.request.GET.get('next'):
            return self.request.GET.get('next')
        return super(CreateReservation, self).get_success_url()

    def get_success_view(self):
        response = HttpResponseRedirect(self.get_success_url())
        response.set_cookie("reservation", str(self.object.pk))
        return response

    def get_form_class(self):
        form = CreateView.get_form_class(self)

        form.request = self.request
        return form

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.user = self.request.user
        self.object.save()
        if TOKENIZE:
            ReservationToken.objects.create(
                reservation=self.object,
                base_url=get_base_url(self.request))
        send_reservation_email(self.object, self.request.user)

        messages.success(self.request, _('Reservation created'))

        return self.get_success_view()


def finish_reservation(request):
    if not hasattr(request, 'reservation'):
        raise Http404(_("No reservation object started"))

    if request.method == "GET":
        response = render(
            request,
            'djreservation/reservation_confirm.html',
            {"reservation": request.reservation})
    elif request.method == "POST":
        reservation = request.reservation
        reservation.status = reservation.REQUESTED
        reservation.save()
        request.reservation = None
        send_reservation_email(reservation, request.user)
        response = render(
            request, 'djreservation/reservation_finished.html')
        response.set_cookie("reservation", "0")
        messages.success(request, _('Reservation finised'))
    return response


class SimpleProductReservation(CreateView):
    model = Reservation
    form_class = ReservationForm
    success_url = "/"
    base_model = None
    amount_field = None
    max_amount_field = None
    extra_display_field = None
    template_name = 'djreservation/simple_reservation.html'
    product_form_class = ProductForm
    product_form = None

    def get_product_form(self, instance, post_context=False):
        self.product_form.initial['model_instance'] = self.pk

        if not post_context:
            self.product_form.initial['amount'] = 1
        if self.max_amount_field and hasattr(instance, self.max_amount_field):
            self.product_form.initial['available_amount'] = getattr(
                instance, self.max_amount_field)
            if post_context:
                self.product_form.fields['available_amount'].value = getattr(
                    instance, self.max_amount_field)
        return self.product_form

    def get_context_data(self, **kwargs):
        context = CreateView.get_context_data(self, **kwargs)
        instance = get_object_or_404(self.base_model, pk=self.pk)
        context['object'] = instance

        context['product_form'] = self.get_product_form(instance)
        return context

    def get_form_kwargs(self):
        kwargs = CreateView.get_form_kwargs(self)
        if END_RESERVATION_DATETIME:
            kwargs['initial']['reserved_end_date'] = END_RESERVATION_DATETIME
        if START_RESERVATION_DATETIME:
            kwargs['initial'][
                'reserved_start_date'] = START_RESERVATION_DATETIME
        return kwargs

    def form_valid(self, form):
        if not self.product_form.is_valid():
            return self.form_invalid(form)

        reservation = form.save(commit=False)
        reservation.user = self.request.user
        reservation.status = Reservation.REQUESTED
        reservation.save()
        self.object = Product(
            amount=self.product_form.cleaned_data['amount'],
            amount_field=self.amount_field,
            reservation=reservation,
            content_object=self.base_instance
        )
        self.object.save()

        if TOKENIZE:
            ReservationToken.objects.create(
                reservation=reservation,
                base_url=get_base_url(self.request)
            )

        send_reservation_email(reservation, self.request.user)
        messages.success(self.request, _('Reservation created successful'))
        return self.get_success_view()

    def get(self, request, *args, **kwargs):

        self.pk = kwargs.pop('pk')
        self.product_form = self.product_form_class()
        return super(SimpleProductReservation, self).get(
            request, *args, **kwargs
        )

    def post(self, request, *args, **kwargs):
        self.pk = kwargs.pop('pk')
        self.product_form = self.product_form_class(request.POST)
        self.base_instance = get_object_or_404(self.base_model, pk=self.pk)
        self.get_product_form(self.base_instance, post_context=True)
        self.product_form.is_valid()
        return CreateView.post(self, request, *args, **kwargs)

    def get_success_view(self):
        return HttpResponseRedirect(self.get_success_url())


class ProductReservationView(CreateView):
    model = Product
    base_model = None
    amount_field = None
    extra_display_field = None
    form_class = ProductForm
    success_url = "/"

    def get_success_view(self):
        return HttpResponseRedirect(self.get_success_url())

    def get_available_amount(self):
        return getattr(self.instance, self.amount_field)

    def get_extra_fields(self):
        extra_fields = []
        for field in self.extra_display_field:
            if hasattr(self.instance, "get_%s_display" % field):
                value = getattr(self.instance, "get_%s_display" % field)()
            else:
                value = getattr(self.instance, field)
            extra_fields.append({
                'label': self.instance._meta.get_field(
                    field).verbose_name,
                'value': value
            })
        return extra_fields

    def get_context_data(self, **kwargs):
        context = CreateView.get_context_data(self, **kwargs)

        context['extra_fields'] = self.get_extra_fields()
        context["instance"] = self.instance
        context["available_amount"] = self.get_available_amount()
        return context

    def form_valid(self, form):
        self.object = Product(
            amount=form.cleaned_data['amount'],
            amount_field=self.amount_field,
            reservation=self.request.reservation,
            content_object=self.instance,

        )
        self.object.save()
        messages.success(self.request, _('Product added successful'))
        return self.get_success_view()

    def get_form_kwargs(self):
        kwargs = CreateView.get_form_kwargs(self)
        kwargs['initial']['model_instance'] = str(self.instance.pk)
        kwargs['initial']["available_amount"] = self.get_available_amount()
        return kwargs

    def get(self, request, *args, **kwargs):
        if "modelpk" in kwargs:
            model_pk = kwargs.pop("modelpk")
            self.instance = self.base_model.objects.get(pk=model_pk)
        return CreateView.get(self, request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        if "modelpk" in kwargs:
            model_pk = kwargs.pop("modelpk")
            self.instance = self.base_model.objects.get(pk=model_pk)
        elif "model_instance" in request.POST:
            self.instance = self.base_model.objects.get(
                pk=request.POST.get("model_instance"))

        return CreateView.post(self, request, *args, **kwargs)


def deleteProduct(request, pk):
    product = get_object_or_404(Product, pk=pk)
    product.delete()
    messages.success(request, _('Product deleted successful'))
    return redirect(reverse("finish_reservation"))


def update_reservation_by_token(request, pk, token, status):
    token_reservation = get_object_or_404(ReservationToken, reservation=pk,
                                          token=token)
    status_available = list(dict(Reservation.STATUS).keys())
    if int(status) not in status_available:
        raise Http404()

    reservation = token_reservation.reservation

    if int(status) == Reservation.ACCEPTED:
        reservation.product_set.all().update(borrowed=True)
    reservation.status = status
    reservation.save()
    token_reservation.delete()
    messages.success(request, _('Reservation updated successful'))
    return redirect("/")



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-05 06:22
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djreservation', '0003_auto_20161004_1525'),
    ]

    operations = [
        migrations.RenameField(
            model_name='reservation',
            old_name='return_date',
            new_name='reserved_end_date',
        ),
    ]



# encoding: utf-8

'''
Free as freedom will be 5/10/2016

@author: luisza
'''

from __future__ import unicode_literals
from django.conf import settings
from django.utils import timezone
from datetime import datetime

TOKENIZE = getattr(settings, 'DJRESERVATION_TOKENIZE', False)

START_RESERVATION_DATETIME = getattr(
    settings, 'DJRESERVATION_START_RESERVATION_DATETIME', None)

END_RESERVATION_DATETIME = getattr(
    settings, 'DJRESERVATION_END_RESERVATION_DATETIME', None)

if START_RESERVATION_DATETIME:
    START_RESERVATION_DATETIME = datetime.strptime(
        START_RESERVATION_DATETIME, '%d/%m/%Y %H:%M')

if END_RESERVATION_DATETIME:
    END_RESERVATION_DATETIME = datetime.strptime(
        END_RESERVATION_DATETIME, '%d/%m/%Y %H:%M')



# encoding: utf-8

'''
Free as freedom will be 26/8/2016

@author: luisza
'''

from __future__ import unicode_literals
from django.views.generic.list import ListView
from django.views.generic.edit import CreateView, DeleteView, UpdateView
from django.conf.urls import url
from django.contrib.auth.decorators import login_required
from django.urls.base import reverse_lazy
from django.utils.decorators import method_decorator
from django.http.response import HttpResponseRedirect


class ObjectView(object):
    model = None
    template_name_base = ""
    namespace = ""
    fields = '__all__'

    def get_base_name(self):
        ns = self.template_name_base
        if not self.template_name_base:
            ns = self.model.__class__.__name__
        return ns

    def get_create_view_class(self):
        return CreateView

    def get_create_view(self):
        CreateViewClass = self.get_create_view_class()

        class OCreateView(CreateViewClass):
            name_urls = self.get_name_urls()

            def get_context_data(self, **kwargs):
                context = super(OCreateView, self).get_context_data(**kwargs)
                context['name_urls'] = self.name_urls
                return context
        return OCreateView

    def get_edit_view_class(self):
        return UpdateView

    def get_edit_view(self):
        EditViewClass = self.get_edit_view_class()

        class OEditView(EditViewClass):
            name_urls = self.get_name_urls()

            def get_context_data(self, **kwargs):
                context = super(OEditView, self).get_context_data(**kwargs)
                context['name_urls'] = self.name_urls
                return context
        return OEditView

    def __init__(self):
        tnb = self.get_base_name()
        OCreateView = self.get_create_view()
        self.create = login_required(OCreateView.as_view(
            model=self.model,
            fields=self.fields,
            success_url=reverse_lazy(self.namespace + ':objectview_list'),
            template_name=tnb + "_form.html"
        ))

        OUpdateView = self.get_edit_view()
        self.edit = login_required(OUpdateView.as_view(
            model=self.model,
            fields=self.fields,
            success_url=reverse_lazy(self.namespace + ':objectview_list'),
            template_name=tnb + "_form.html"
        ))

        self.delete = login_required(DeleteView.as_view(
            model=self.model,
            success_url=reverse_lazy(self.namespace + ':objectview_list'),
            template_name=tnb + "_delete.html"
        ))

        OListView = self.get_list_view()
        self.list = login_required(OListView.as_view(
            model=self.model,
            paginate_by=10,
            template_name=tnb + "_list.html"
        ))

    def get_list_view_class(self):
        return ListView

    def get_list_view(self):
        OListViewClass = self.get_list_view_class()

        class OListView(OListViewClass):
            name_urls = self.get_name_urls()

            def get_context_data(self, **kwargs):

                context = super(OListView, self).get_context_data(**kwargs)
                context['name_urls'] = self.name_urls
                return context
        return OListView

    def get_name_urls(self):
        return {
            'add': self.namespace + ":objectview_create",
            'edit': self.namespace + ":objectview_update",
            'delete': self.namespace + ":objectview_delete",
            'list': self.namespace + ":objectview_list"
        }

    def get_urls(self):
        return [
            url(r"^list$", self.list, name="objectview_list"),
            url(r"^create$", self.create, name="objectview_create"),
            url(r"^edit/(?P<pk>\d+)$", self.edit, name="objectview_update"),
            url(r"^delete/(?P<pk>\d+)$", self.delete,
                name="objectview_delete"),
        ]


class UserObjectView(ObjectView):

    def get_create_view_class(self):
        class UCreateView(CreateView):

            def form_valid(self, form):
                self.object = form.save(commit=False)
                self.object.user = self.request.user
                self.object.save()
                return HttpResponseRedirect(self.get_success_url())
        return UCreateView

    def get_edit_view_class(self):
        class UUpdateView(UpdateView):

            def form_valid(self, form):
                self.object = form.save(commit=False)
                self.object.user = self.request.user
                self.object.save()
                return HttpResponseRedirect(self.get_success_url())
        return UUpdateView

    def get_list_view_class(self):
        class UListView(ListView):

            def get_queryset(self):
                queryset = super(UListView, self).get_queryset()
                queryset = queryset.filter(user=self.request.user)
                return queryset
        return UListView



from django.shortcuts import render

from djreservation.views import ProductReservationView
from .models import MyObject


class MyObjectReservation(ProductReservationView):
    base_model = MyObject     # required
    amount_field = 'quantity'  # required
    extra_display_field = ['measurement_unit']  # not required


def home(request):
    list_object = MyObject.objects.all()
    return render(request, 'index.html', context={
        'list_object': list_object
    })
